<?php
/**
* @copyright	Copyright (C) 2009 - 2013 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	Frontend
* @contact 		payplans@readybytes.in
* 
*/
if(defined('_JEXEC')===false) die();

class PayplansAppPaywithpoints extends PayplansAppPayment
{
	protected $_location	= __FILE__;
	
	
	// applicable only if aup exist
	public function _isApplicable(PayplansIfaceApptriggerable $refObject, $eventName='')
	{	
		$pointType = $this->getAppParam('pointsType','');
		if($pointType == 'aup')
			return JFolder::exists(JPATH_SITE .DS.'components'.DS.'com_altauserpoints');
		if($pointType == 'karma')
			return JComponentHelper::isEnabled('com_community', true );
		if($pointType == 'both')
			return (JFolder::exists(JPATH_SITE .DS.'components'.DS.'com_altauserpoints')&&
							JComponentHelper::isEnabled('com_community', true ));
		
	}
	public function onPayplansPaymentForm(PayplansPayment $payment, $data = null)
	{
		$invoice 	   = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$amount  	   = $invoice->getTotal();
		$pointType     = $this->getAppParam('pointsType','');
		$points        = $this->_getPoints($invoice);
			
		$this->assign('amount', $amount);
		$this->assign('deductingPoints', $this->_getAppPoints());
		$this->assign('invoice', $invoice);
		$this->assign('payment_key', $payment->getKey());
		$this->assign('point_type', $pointType);
		$this->assign('points', $points);
		$this->assign('posturl',	XiRoute::_('index.php?option=com_payplans&view=payment&task=complete&payment_key='.$payment->getKey()));
        return $this->_render('form');
	}
	
	
	protected function _getPoints($invoice)
	{
		$pointType = $this->getAppParam('pointsType','');
		$buyer= $invoice->getBuyer();
      	if($pointType == 'aup'){
      	  require_once (JPATH_SITE.DS.'components'.DS.'com_altauserpoints'.DS.'helper.php');
		  $aupid = AltaUserPointsHelper::getAnyUserReferreID($buyer);
		  $userInfo = AltaUserPointsHelper::getUserInfo($aupid);
		  $points = array($userInfo->points);
		  
		}
		elseif($pointType == 'karma'){
		  require_once(JPATH_ROOT.DS.'components'.DS.'com_community'.DS.'libraries'.DS.'core.php');
		  $this->_user   = CFactory::getUser($buyer);
		  $points = array($this->_user->getKarmaPoint());
		  
		}
		else { 
		  require_once (JPATH_SITE.DS.'components'.DS.'com_altauserpoints'.DS.'helper.php'); 
          require_once(JPATH_ROOT.DS.'components'.DS.'com_community'.DS.'libraries'.DS.'core.php');
		  $this->_user   = CFactory::getUser($buyer);
		  $aupid = AltaUserPointsHelper::getAnyUserReferreID($buyer);
		  $userInfo = AltaUserPointsHelper::getUserInfo($aupid);
		  $points = array('aup'=>$userInfo->points, 'karma'=>$this->_user->getKarmaPoint());
		}
		return $points;
	}
	protected function _getAppPoints()
	{
		$pointType = $this->getAppParam('pointsType','');
		if($pointType == 'aup')
			return $this->getAppParam('aup_points');
		if($pointType == 'karma')
			return $this->getAppParam('karma_points');
		if($pointType == 'both')
			return $this->getAppParam('aup_karma_points');
		
	}
	
	public function onPayplansPaymentAfter(PayplansPayment $payment, &$action, &$data, $controller)
	{
		if($action == 'cancel')
		   return ;

		$invoice  = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$amount	  = $invoice->getTotal();

		$errors = $this->_checkPaymentRequest($payment, $amount,$data);
					
		if($errors && !empty($errors)){
			$action = 'error';
			return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
		}
	
		if($action == 'success'){
			$this->_processTransaction($payment, $data);						
			return true;
		}
	}
	
	//before subscription expiry process the future payment if allowed
	public function processPayment(PayplansPayment $payment, $invoiceCount)
	{
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$invoice_count = $invoiceCount + 1;
		
		if($invoice->isRecurring())
		{
			$recurrence_count 	= $payment->getGatewayParam('pending_recur_count');

			if($recurrence_count > 0)
			{
				$amount = $invoice->getTotal($invoice_count);
				$errors = $this->_checkPaymentRequest($payment, $amount);
				//if any error exists then do not create transaction
				if(!empty($errors)){
					return true;
				}
				
				$pendingRecurrence = $this->_getRecurrenceCount($invoice) - $invoice_count;
				$data = array();
				$this->_processTransaction($payment, $data, $pendingRecurrence);

				return true;
			}
		}
	}
	

	protected function _checkPaymentRequest(PayplansPayment $payment, $amount,$data)
	{
		$invoice  = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$user     = $invoice->getBuyer(PAYPLANS_INSTANCE_REQUIRE);
		$points   = $this->_getPoints($invoice);
		if(isset($data['aup_or_karma']))
		{
			$userPoints=$points[$data['aup_or_karma']];
		}
		else 
		{
			$userPoints= array_shift($points);
		}
		if(defined('PAYPLANS_UNIT_TEST_MODE')===true){
				return true;
			}
		$errors = array();
		
		
		//	amount in wallet is insufficient for the invoice
		if($this->_getAppPoints() > $userPoints){
			$errors['error_message'] 	= XiText::_('COM_PAYPLANS_PAY_WITH_POINTS_INSUFFICIENT_POINTS');
			$errors['response']			= $data; 
			$message 					= XiText::_('COM_PAYPLANS_LOGGER_ERROR_IN_PAY_WITH_POINTS_PAYMENT_PROCESS');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $errors, 'PayplansPaymentFormatter', '', true);
		}
		
		return $errors;
	}
	
	protected function _processTransaction(PayplansPayment $payment, $data, $pendingRecurrence=null)
	{
		$invoice  = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		
		//apply 100% discount
		$modifier = PayplansModifier::getInstance();
		$modifier->set('message', XiText::_('COM_PAYPLANS_PAY_WITH_POINTS_MESSAGE' ))
				 ->set('invoice_id', $invoice->getId())
				 ->set('user_id', $invoice->getBuyer())
				 ->set('type', 'pay_with_points')
				 ->set('amount', -100) // 100 percent Discount, discount must be negative
				 ->set('percentage', true) 
				 ->set('frequency', PayplansModifier::FREQUENCY_ONE_TIME)
				 ->set('serial', PayplansModifier::PERCENT_DISCOUNT)
				 ->save();			  
		$invoice->refresh()->save();
		
		$transaction = PayplansTransaction::getInstance();
		$transaction->set('user_id', $invoice->getBuyer())
					->set('invoice_id', $invoice->getId())
					->set('payment_id', $payment->getId())
					->set('amount', 0)
					->set('gateway_txn_id', 0)
					->set('gateway_subscr_id', 0)
					->set('gateway_parent_txn', 0)
					->set('message', 'COM_PAYPLANS_APP_PAY_WITH_POINTS_TRANSACTION_CREATED_FOR_INVOICE')
					->save();

		//update recurrence count in payment gateway params
		if($invoice->isRecurring()){
			if($pendingRecurrence== null){
				$pendingRecurrence = $this->_getRecurrenceCount($invoice)-1;
			}
			$params = array('pending_recur_count' => $pendingRecurrence);
			$payment->set('gateway_params', PayplansHelperParam::arrayToIni($params))
						->save();
		}
	
		$this->_deductPoints($data,$invoice);
		return true;
	}
	protected function _deductPoints($data,$invoice)
	{
		$buyer = $invoice->getBuyer();
		$pointType = $this->getAppParam('pointsType','');
		if($pointType == 'aup')
		{   
			$aupPoints = $this->getAppParam('aup_points');
			$aupid = AltaUserPointsHelper::getAnyUserReferreID( $buyer);
			//this should use the aup methods for adding points, it should bypass any level restrictions etc...
        	AltaUserPointsHelper::newpoints( 'paywithpoints', $aupid, '', '', -$aupPoints, '', 1);
			return;
		}
		if($pointType == 'karma')
		{  
			$karmaPoints = $this->getAppParam('karma_points');
			$this->_user->_points = $this->_user->getKarmaPoint()-$karmaPoints;
			$this->_user->save();
			return;
		}	
		
		if($pointType == 'both')
		{
			$selected = $data['aup_or_karma'];
			$points = $this->getAppParam('aup_karma_points');

			if($selected == 'karma')
			{
				$this->_user->_points = $this->_user->getKarmaPoint()-$points;
				$this->_user->save();
				return;
			}
			
			$aupid = AltaUserPointsHelper::getAnyUserReferreID( $buyer);
			//this should use the aup methods for adding points, it should bypass any level restrictions etc...
        	AltaUserPointsHelper::newpoints( 'paywithpoints', $aupid, '', '', -$points, '', 1);
		
		}
		
	}
	protected function _getRecurrenceCount(PayplansInvoice $invoice)
	{
		$count = $invoice->getRecurrenceCount();
		// for lifetime recurrence
		if(intval($count) === 0){
			return 9999;
		}
		
		$recurring = $invoice->isRecurring();
		if($recurring){
			
			// Recurrence Count For Regular Recurring Plan
       		if($recurring == PAYPLANS_RECURRING){
	       		$recurrence_count = $count;
       		}
       		// Recurrence Count For Recurring + Trial 1 Plan
       		if($recurring == PAYPLANS_RECURRING_TRIAL_1){
				$recurrence_count = $count + 1;
       		}
       		// Recurrence Count For Recurring + Trial 2 Plan
       		if($recurring == PAYPLANS_RECURRING_TRIAL_2){
	       		$recurrence_count = $count + 2;
       		}
		}
		return $recurrence_count;
	}
	
	public function onPayplansPaymentTerminate(PayplansPayment $payment, $controller)
	{
		return $this->_render('cancel');
	}

	}
    
